mod lib_test;

use serde::Deserialize;
use trust_dns_resolver::AsyncResolver;
use reqwest;
use std::io::{
  Error,
  ErrorKind
};

#[derive(Deserialize)]
pub struct TokenServiceApi {
  pub main:         String,
  pub youtube:      String,
  pub octokit:      String,
  pub postgres_uri: String,
  pub redis_uri:    String,
  pub wg_pms:       String
}

pub struct TokenService {
  endpoint: String,
  server: String
}

impl TokenService {
  pub fn new(endpoint: &str) -> Self {
    TokenService {
      endpoint: endpoint.to_string(),
      server: "ts.local.toast-data.center".to_string()
    }
  }

  pub async fn connect(&self) -> Result<TokenServiceApi, Box<dyn std::error::Error + Send + Sync>> {
    let resolver = match AsyncResolver::tokio_from_system_conf() {
      Ok(resolver) => resolver,
      Err(y) => return Err(Box::new(Error::new(ErrorKind::Other, format!("Failed to create resolver: {}", y))))
    };

    let _ = resolver;
    let url = format!("http://{}/{}", self.server, self.endpoint);
    let response = match reqwest::get(&url).await {
      Ok(response) => response,
      Err(y) => return Err(Box::new(Error::new(ErrorKind::Other, format!("Failed to send GET request to {}: {}", url, y))))
    };

    if !response.status().is_success() {
      return Err(Box::new(Error::new(ErrorKind::Other, format!("Received an error response code from server: {}", response.status()))))
    };

    let api: TokenServiceApi = match response.json().await {
      Ok(api) => api,
      Err(err) => return Err(Box::new(Error::new(ErrorKind::Other, format!("Failed to deserialize response into TokenServiceApi: {}", err))))
    };

    Ok(api)
  }
}
